UNIT HRange;

INTERFACE

USES Linlib,GlobType;


FUNCTION Schoener(n : Word; VAR X,Y : Arr) : Single;
FUNCTION Schoener2(n : Word;MeanX,MeanY : Single;VAR X,Y : Arr) : Single;
FUNCTION JennrichT(n,Level : Word; VAR X,Y : Arr): Extended;
PROCEDURE JennrichTGeneral(    n,Level          : Word;
                           VAR X,Y              : Arr;
                           VAR Area,MeanX,MeanY,
                               CorrXY           : Extended;
                           VAR a,b,Theta        : Single);



IMPLEMENTATION

{Computes Schoener's ratio, note: Sum(Sqr(xi-xi+1) + Sqr(yi - yi+1))/(n-1),
 i = 1,n is = Sum(Sqr(xi-xi+1))/(n-1) + Sum(Sqr(yi - yi+1))/(n-1) - which is
 how TSq and Rsq are computed here.}


{distance squared between members of either x or y}

FUNCTION SeqSquare(n : Word;VAR Pts : Arr):Double;
         VAR
            Ct : Word;
            Sum : Double;
         BEGIN
              Sum := 0;
              FOR Ct :=1 TO (n - 1) DO
                  Sum := Sum + Sqr(Pts[Ct+1] - Pts[Ct]);
              SeqSquare := Sum/(n - 1);
END;

{distance squared between members of either x or y and the arithmetic mean
 of either x or y}

FUNCTION MeanSquare(n : Word;ArithMean : Single;VAR Pts : Arr):Double;
         VAR
            Ct : Word;
            Sum : Double;
         BEGIN
              Sum := 0;
              FOR Ct :=1 TO n DO
                  Sum := Sum + Sqr(Pts[Ct] - ArithMean);
              MeanSquare := Sum/(n - 1);
END;
FUNCTION Schoener(n : Word; VAR X,Y : Arr) : Single;
         VAR
            MeanX,MeanY : Single;
            TSq,RSq     : Double;
         BEGIN
              MeanX := Mean(n,X);    {get means}
              MeanY := Mean(n,Y);
              TSq   := SeqSquare(n,X) + SeqSquare(n,Y);
              RSq   := MeanSquare(n,MeanX,X) + MeanSquare(n,MeanY,Y);
              Schoener := TSq/RSq;
END;

{Schoener2 does not compute the mean, use where you need to compute this value
 externally}

FUNCTION Schoener2(n : Word;MeanX,MeanY : Single;VAR X,Y : Arr) : Single;
         VAR
            TSq,RSq     : Double;
         BEGIN
              TSq   := SeqSquare(n,X) + SeqSquare(n,Y);
              RSq   := MeanSquare(n,MeanX,X) + MeanSquare(n,MeanY,Y);
              Schoener2 := TSq/RSq;
END;

{Procedure Jennrich computes the bivariate ellipse area
 for vectors X and Y}

FUNCTION JennrichT(n,Level : Word; VAR X,Y : Arr) : Extended;
          CONST
               ChiSquare : Array[1..7] OF Single =
                           (2.77,4.61,5.99,7.38,9.21,10.6,13.8);
            {Critical pts.  = 0.25,0.10,0.05,0.025,0.01,0.005,0.001}
          VAR
             Ct                      : Word;
             MeanX,MeanY,SSx,SSy,Sxy : Extended;
          BEGIN
               IF (Level < 1) OR (Level > 7) THEN BEGIN
                  WriteLn('Error - improper X input, confidence set to .05');
                  Level := 3;
               END;
               MeanX := Mean(n,X);
               MeanY := Mean(n,Y);
               SSx := 0;
               FOR Ct := 1 TO n DO
                   SSx := SSx + Sqr(X[Ct] - MeanX);
               SSx := SSx/(n-2);
               SSy := 0;
               FOR Ct := 1 TO n DO
                   SSy := SSy + Sqr(Y[Ct] - MeanY);
               SSy := SSy/(n-2);
               Sxy := 0;
               FOR Ct := 1 TO n DO
                   Sxy := Sxy + (Y[Ct] - MeanY) * (X[Ct] - MeanX);
               Sxy := Sxy/(n-2);
               JennrichT := Pi * Sqrt(SSx*SSy - Sqr(Sxy)) * ChiSquare[Level];
END;

{JennrichTGeneral computes all of the pertinent data that can be associated
 with a Jennrich Turner estimator:

 Area        = Area estimate;
 a,b         = the foci
 MeanX,MeanY = the arithmetic mean X,Y
 Theta       = the angle of inclination
 CorrXY      = the correlation between X,Y

 }

PROCEDURE JennrichTGeneral(    n,Level          : Word;
                           VAR X,Y              : Arr;
                           VAR Area,MeanX,MeanY,
                               CorrXY           : Extended;
                           VAR a,b,Theta        : Single);

          CONST
               ChiSquare : Array[1..7] OF Single =
                           (2.77,4.61,5.99,7.38,9.21,10.6,13.8);
            {Critical pts.  = 0.25,0.10,0.05,0.025,0.01,0.005,0.001}
          VAR
             Ct                      : Word;
             SSx,SSy,Sxy,R           : Extended;
          BEGIN
               IF (Level < 1) OR (Level > 7) THEN BEGIN
                  WriteLn('Error - improper X input, confidence set to .05');
                  Level := 3;
               END;
               MeanX := Mean(n,X);
               MeanY := Mean(n,Y);
               SSx := 0;
               FOR Ct := 1 TO n DO
                   SSx := SSx + Sqr(X[Ct] - MeanX);
               SSx := SSx/(n-2);
               SSy := 0;
               FOR Ct := 1 TO n DO
                   SSy := SSy + Sqr(Y[Ct] - MeanY);
               SSy := SSy/(n-2);
               Sxy := 0;
               FOR Ct := 1 TO n DO
                   Sxy := Sxy + (Y[Ct] - MeanY) * (X[Ct] - MeanX);
               Sxy := Sxy/(n-2);
               Area   := Pi * Sqrt(SSx*SSy - Sqr(Sxy)) * ChiSquare[Level];
               CorrXY := Sxy/Sqrt(SSx*SSy);
               R      := Sqrt(Sqr(SSy - SSx) + 4 * Sqr(Sxy));
               a      := Sqrt(((SSy + SSx + R) * ChiSquare[Level])* 0.5);
               b      := Sqrt(((SSy + SSx - R) * ChiSquare[Level])* 0.5);
               Theta  := ArcTan((-2 * Sxy)/(SSy - SSx - R));
END;

BEGIN
END.