
{MemHand provides 3 simple procedures that have great power for memory
 management.  By using CreateArray and DestroyArray rather than
 New and Dispose you can create arrays of arbitrary size on the fly -
 you can violate the formal dimensions of the array.  ChangeArray
 allows you to redimension an array on the fly - if you have an
 array-sized chunk of swap space.  As an example, lets say that
 you are reading an element from a file into array X. How big should
 array X be?  Using this approach you can simply increase the size of
 X whenever the its space is exceeded (if DataCtr > SpaceAllocated then
 use ChangeArray to increase the space by, say 500 elements).  When the
 file is finally read, you can shrink X to the exact dimensions of the
 data set using ChangeArray.  Subsequent arrays can then be dimensioned
 to DataCtr - and freely created and destroyed using CreateArray and
 DestroyArray.  Notice that for file testing, unless DataCtr exceeds
 the formal dimesnion of the array, that the range checks can be used
 to debug the program.}


Unit MemHand;

Interface

uses utility;

Function CreateArray(n,ElementSize : Word;VAR ArrPtr   : Pointer):Boolean;
Procedure DestroyArray(n,ElementSize : Word;VAR ArrPtr : Pointer);
Function ChangeArray(     OldSize,NewSize,TransSize,
                          ElementSize                  : Word;
                      VAR ArrPtr                       : Pointer): Boolean;

implementation

{Create array allocates just enough memory to hold n elements.  It returns
 TRUE if any errors occured, FALSE otherwise}

Function CreateArray(n,ElementSize : Word;VAR ArrPtr : Pointer):Boolean;
          VAR
             Size : LongInt;
          Begin
               Size := n * ElementSize;
               if (MaxAvail > Size) and (Size < 65520) then begin
                  GetMem(ArrPtr,Size);
                  CreateArray := FALSE;
               end
               Else begin
                    ArrPtr := Nil;
                    CreateArray := TRUE;
               end;
end;

{DestroyArray will deallocate n * elementSize chunks of memory}

Procedure DestroyArray(n,ElementSize : Word;VAR ArrPtr : Pointer);
          VAR
             Size : Word;
          Begin
               if ArrPtr <> NIL then begin
                  FreeMem(ArrPtr,Size);
                  ArrPtr := NIL;
               end;
end;

{ChangeArray creates a space on the heap big enough to hold NewSize
 elements.  It then transfers TransSize elements to the new area and
 deallocates OldSize * elementSize elements. Lastly, ArrPtr is pointed to
 the new area. You can use ChangeArray to either swell or shrink an array.}

Function ChangeArray(     OldSize,NewSize,TransSize,
                          ElementSize                  : Word;
                      VAR ArrPtr                       : Pointer): Boolean;
          VAR
             Size    : LongInt;
             TempPtr : Pointer;
          begin
               Size := NewSize * ElementSize; 
               if (MaxAvail > Size) and (Size < 65520) then begin
                  GetMem(TempPtr,Size);              {Set up a new area}
                  Size := TransSize * ElementSize;
                  FastMove(ArrPtr^,TempPtr^,Size);   {transfer to it}
                  Size := OldSize * ElementSize;
                  FreeMem(ArrPtr,Size);              {destroy old area}
                  ArrPtr := TempPtr;                 {point old pointer at}
                  ChangeArray := FALSE;              {new area}
               end
               else
                   ChangeArray := TRUE;
end;
begin
end.